#ifndef _DISPLAY_																//    
#define _DISPLAY_

#define LED_OFF 0																/* " "*/
#define LED_ON	1																/* " "*/

#ifdef _DISPLAY_ENABLE_															//  

#ifndef F_CPU
# warning "F_CPU not defined. Frequency by default 16MHz"
#define F_CPU 16000000UL														/*     (     )*/
#endif

#include <avr/iom128.h>															//  /
#include <util/delay.h>															//   

static char _Display_buf[4];													//      SPI

//=====================       SPI ========================
void DisplayOut(void)
{
	__asm__ __volatile__ ("cli");												//  

#ifdef _NLCON1ATD3_																//  	
	PORTF&=~(1<<PF7);															//   
#endif
#ifdef _NLCON1AT2D3_															//  	
	PORTE&=~(1<<PE2);															//   
#endif

	SPDR=_Display_buf[0];														//      
	while(!(SPSR & (1<<SPIF)));													//   
	SPSR=0;																		//  

#ifdef _NLCON1ATD3_																//  
	PORTF|=(1<<PF7);															//   
#endif
#ifdef _NLCON1AT2D3_															//  
	PORTE|=(1<<PE2);															//   
#endif

	_delay_us(10);																//   

#ifdef _NLCON1ATD3_																//  
	PORTF&=~(1<<PF7);															//   
#endif
#ifdef _NLCON1AT2D3_															//  
	PORTE&=~(1<<PE2);															//   
#endif

	SPDR=_Display_buf[1];														// 1     
	while(!(SPSR & (1<<SPIF)));													//   
	SPDR=_Display_buf[2];														// 2     
	while(!(SPSR & (1<<SPIF)));													//   
	SPDR=_Display_buf[3];														// 3     
	while(!(SPSR & (1<<SPIF)));													//   

#ifdef _NLCON1ATD3_																//  
	PORTF|=(1<<PF7);															//   
#endif
#ifdef _NLCON1AT2D3_															//  
	PORTE|=(1<<PE2);															//   
#endif
	
	__asm__ __volatile__ ("sei");												//  
}

//==========================    =============================
void DisplayInit(void)
{
	DDRB|=(1<<PB0)|(1<<PB1)|(1<<PB2)|(1<<PB3);									//  SPI  
	SPSR=0x00;																	//   SPI
	SPCR=(1<<SPE)|(1<<MSTR)|(1<<CPOL)|(1<<CPHA)|(1<<SPR0);						// SPI (F_CPU/16)

#ifdef _NLCON1ATD3_																//  
	DDRF|=(1<<PF7);																//     
	PORTF|=(1<<PF7);															//   
#endif
#ifdef _NLCON1AT2D3_															//  
	DDRE|=(1<<PE2);																//     
	PORTE|=(1<<PE2);															//   
#endif	
}

//=====================       ========================
//    :
//data_buf -        (   ,     ( ),   )

void DisplayStr(char *data_buf)
{
	static int temp,temp2;														// 
	static char Sign;															//    +  -
	static char Point;															//     ()
	static int Digit;															//  
	static char ShiftDigit;														//    (         1)
	
	static char buf[5];															//     
	
	temp2=0;
	Sign=0;
	Point=0;
	Digit=0;
	ShiftDigit=0;
	
	for (temp=0; temp<2; temp++)												//     (   +  - ,    )
	{
		if (data_buf[temp]=='+')												//  + 
		{
			Sign|=0x01;
			temp2++;
		}
		if (data_buf[temp]=='-')												//  - 
		{
			Sign|=0x02;
			temp2++;
		}
	}
	
	for (temp=0; temp<5; temp++) buf[temp]=0;									//  
	
	for (temp=temp2; (data_buf[temp]!=0) && (Digit<5); temp++)					//  ,    - (  )       
	{
		if (temp==temp2)														//       +,-
		{
			if ((data_buf[temp]<'0') || (data_buf[temp]>'1')) ShiftDigit=1;		//      0  1,      
			if ((data_buf[temp]=='.') || (data_buf[temp]==','))					//     
			{
				Point=1;														//   
				ShiftDigit=0;													//    
				Digit++;														//   .   ( )  
				continue;														//    
			}
		}

		//     0   9
		if ((data_buf[temp]>='0') && (data_buf[temp]<='9')) buf[Digit++]=data_buf[temp]-0x30;
		else
			//     A   F
			if ((data_buf[temp]>='A') && (data_buf[temp]<='F')) buf[Digit++]=data_buf[temp]-0x37;
			else
				//     a   f
				if ((data_buf[temp]>='a') && (data_buf[temp]<='f')) buf[Digit++]=data_buf[temp]-0x57;
				else
					//     
					if (((data_buf[temp]=='.') || (data_buf[temp]==',')) && (Point==0)) Point=Digit;
	}
	//     
	if (((data_buf[temp]=='.') || (data_buf[temp]==',')) && (Point==0)) Point=Digit;
	if (Digit>4) Digit-=ShiftDigit;												//     4,    
	buf[Digit]=0;																//    (0-  )

	temp=Digit;																	//    
	
	_Display_buf[0]=0xC1;														//      
	_Display_buf[1]=0x80;
	_Display_buf[2]=0;
	_Display_buf[3]=0;
	
	if (temp!=0)
	{
		_Display_buf[1]|=buf[--temp];												//   1  
		if (temp!=0)
		{
			_Display_buf[2]|=(buf[--temp]<<4);										//   2  
			if (temp!=0)
			{
				_Display_buf[2]|=buf[--temp];										//   3  
				if (temp!=0) _Display_buf[3]|=(buf[--temp]<<4);						//   4  
			}
		}
	}

	if ((Digit<5) || ((Digit==5) && (buf[0]==0)))								//  1    0     (..    )
	{
		switch (Sign)
		{
			case 0:
				_Display_buf[0]|=0x02;	//										//  
				break;
			case 1:
				_Display_buf[3]|=0x0C;	//+										//     +
				break;
			case 2:
				_Display_buf[0]|=0x02;	//-										//     -
				_Display_buf[3]|=0x01;
				break;
			case 3:
				_Display_buf[3]|=0x0E;	//+-									//    +  -
		}
	}
	else																		//       1
	{
		switch (Sign)
		{
			case 0:
				_Display_buf[3]|=0x01;	//1										//    1
				break;
			case 1:						//+1									//   +1
				break;
			case 2:
				_Display_buf[3]|=0x0D;	//-1									//   -1
				break;
			case 3:
				_Display_buf[3]|=0x04;	//+-1									//   +-1
		}
	}
	
	ShiftDigit=0xF0<<Digit;
	ShiftDigit>>=Digit+2;
	_Display_buf[0]|=ShiftDigit;
	
	if (Point!=0)																//   
	{
		Point+=(5-Digit);														//   
		_Display_buf[1]|=Point<<4;												//    
	}
	
	DisplayOut();																//      SPI
}

//====================       =======================
//    :
//data -        
//digit -     .        ,     
//point -     (        ,    )
// :
//0x00-      ,    0xFF

char DisplayDec(int data, int digit, unsigned int point)
{
	static int temp,temp2;														// 
	static char str_temp[8];													//       
	static int ShiftPoint;														//   
		
	for (temp=0; temp<8; temp++) str_temp[temp]=0;								//    
	
	if (((data<0) && (data<-19999)) || ((data>0) && (data>19999))) return 0xFF;	//       ,   
	
	if (data!=0)																//    
	{
		if (data>0) temp2=data;													//   ,  
		else temp2=-data;
		
		//   
		for (ShiftPoint=0; (ShiftPoint<5) && ((temp2!=0) || (digit>0)); ShiftPoint++, digit--)	
		{
			str_temp[5-ShiftPoint]=(temp2 % 10)+0x30;							//   
			temp2/=10;															//  
		}
		ShiftPoint++;															//  
		
		while (str_temp[1]==0)													//     
		{
			for (temp=2; temp<7; temp++) str_temp[temp-1]=str_temp[temp];		//   
		}
		
		if (data<0) str_temp[0]='-';											//     
		else str_temp[0]='+';													//  
		
		if ((ShiftPoint-point)>1)												//       ,   
		{
			//  
			for (temp=6; temp>(ShiftPoint-point); temp--) str_temp[temp]=str_temp[temp-1];
			str_temp[temp]='.';													// 
		}
		
		DisplayStr(str_temp);													//     
	}
	else DisplayStr("0");														//    
	
	return 0;																	//   
}

//=================       ===================
//    :
//data -        
// :
//0x00-    ,    0xFF

void DisplayHex(unsigned int data)
{
	static int temp,temp2;														// 
	static char str_temp[6];													//       
		
	for (temp=0; temp<6; temp++) str_temp[temp]=0;								//    
	
	for (temp=0; temp<4; temp++)												//   
	{
		temp2=data % 16;
		if (temp2<10) str_temp[3-temp]=temp2+0x30;								//    (0-9)
		else str_temp[3-temp]=temp2+0x37;										//    (A-F)
		data/=16;																//  
	}

	DisplayStr(str_temp);														//     
}

//===================       ======================
//    :
//dt -     
//    ,        
// :
//0x00-        ,    0xFF

char DisplayFloat(float data)
{
	static int integer;															//     
	static int point;															//    
	static int temp,temp2,temp3;												// 
	static char str_temp[8];													//       
	static float Float;															//    
	
	//       ,   
	if (((data<0) && (data<=-20000)) || ((data>0) && (data>=20000))) return 0xFF;
	
	for (temp=0; temp<8; temp++) str_temp[temp]=0;								//    
	
	Float=data;																	//      
	if (data<0) data=-data;														//  ,  
	
	integer=data;																//       
	temp=0;																		//    

	while ((integer<1) && (integer!=0))											//    1 (        )
	{
		data*=10;																//     1  
		integer=data;															//       
		temp--;																	//  
	}

	while ((integer>=10) && (integer!=0))										//    10 (        )
	{
		data/=10;																//     1  
		integer=data;															//       
		temp++;																	//  
	}

	point=temp;																	//  
	if (temp>=0) point++;														//  , ..        
	
	//      1,         .      
	if (integer>1) point++;														
	
	data=Float;																	//    
	integer=data;																//       
	
	//""       (     )
	while (((integer<2000) && (integer>0)) || ((integer>-2000) && (integer<0)))
	{
		data*=10;																//  ( 1  )
		
		if (integer>0) integer=data+0.5;										//    ( )
		else integer=data-0.5;
	}
	
	if (integer<0)																//  
	{
		str_temp[0]='-';														//    
		integer=-integer;														//  
	}
	else str_temp[0]='+';														//  ,     
	
	for (temp=0; temp<5; temp++)												//    
	{
		str_temp[5-temp]=(integer % 10)+0x30;									//     
		integer/=10;															//     (  1  )
	}
	
	for (temp=0, temp2=point; temp2<0; temp2++, temp++)							//      (    )
	{
		for (temp3=6; temp3>1; temp3--) str_temp[temp3]=str_temp[temp3-1];		//     1  
		str_temp[1]='0';														//    
	}
	
	if (point<0) point=1;														//      
	for (temp=6; temp>point; temp--) str_temp[temp]=str_temp[temp-1];			//    
	str_temp[point+1]='.';														// 

	DisplayStr(str_temp);														//     
	return 0;																	//   
}
#endif

//============================    ==============================
//    :
//Enable -     (1-, 0-).

void Led(char power)
{
	DDRG|=0x10;																	//   
	if (power) PORTG|=0x10;														// 
	else PORTG&=0xEF;															//  
}
#endif
